@@ -41,11 +41,10 @@ gene2pathway.signaltrans = function(geneIDs=NULL, flyBase=FALSE, gene2Domains=NU
 		cat("Mapping to signal transduction pathway components via KEGG database ...\n",sep="")
 		if(!(organism %in% c("hsa", "mmu", "rno"))){
 			if(flyBase){
-				ensembl <- useMart("ensembl", dataset = "dmelanogaster_gene_ensembl")
-				tmp <- getBM(attributes=c("flybase_gene_id", "entrezgene"), filters="flybase_gene_id", values=geneIDs, mart = ensembl)
-				if(is.null(geneIDs))
+				geneIDs = unlist(AnnotationDbi::mget(geneIDs, org.Dm.egFLYBASE2EG, ifnotfound=NA))
+				geneIDs = geneIDs[!is.na(geneIDs)]
+				if(length(geneIDs) == 0)
 					stop("No mapping FlyBase -> Entrez gene ID found!")
-				geneIDs = geneIDs[,2]
 				flyBase = FALSE			
 			}
 			KEGG2Entrez.tab = gene2pathway:::KEGG2Entrez(organism=organism)
@@ -121,7 +120,12 @@ gene2pathway.signaltrans = function(geneIDs=NULL, flyBase=FALSE, gene2Domains=NU
 		})				
 	})
 
-	kegg_hierarchy = gene2pathway:::getKEGGHierarchy(level1Only=c(), level2Only=c())
+	if(exists("kegg_hierarchy", envir=gene2pathwayEnv))
+		kegg_hierarchy = get("kegg_hierarchy", envir=gene2pathwayEnv)
+	else{
+		kegg_hierarchy = gene2pathway:::getKEGGHierarchy(level1Only=c(), level2Only=c())
+		assign("kegg_hierarchy", kegg_hierarchy, envir=gene2pathwayEnv)		
+	}
 	pathnames = c(kegg_hierarchy$pathNamesLev1, kegg_hierarchy$pathNamesLev2, kegg_hierarchy$pathNamesLev3)		
 	totallist[totallist == ""] = NA
 	totallist[is.na(names(totallist))] = NA
